@@ -1,6 +1,10 @@
 package LISM::Handler;
 
 use strict;
+use URI;
+use Net::LDAP;
+use Net::LDAP::Constant qw(:all);
+use Encode;
 use Sys::Syslog;
 use Sys::Syslog qw(:macros);
 use Data::Dumper;
@@ -301,6 +305,10 @@ sub log
     my $conf = $self->{_config};
     my %p = @_;
 
+    if (Encode::is_utf8($p{'message'})) {
+        $p{'message'} = encode('utf8', $p{'message'});
+    }
+
     openlog('LISM', 'pid', 'local4');
     setlogmask(Sys::Syslog::LOG_UPTO($conf->{numloglevel}{$conf->{sysloglevel}}));
     syslog($conf->{numloglevel}{$p{'level'}}, $p{'message'});
@@ -312,6 +320,91 @@ sub _checkConfig
     return 0;
 }
 
+sub _parseLdapUri
+{
+    my $self = shift;
+    my ($ldapopts, $ldapuri) = @_;
+
+    $ldapopts->{uri} = $ldapuri;
+    my $uri = URI->new($ldapopts->{uri});
+    $ldapopts->{base} = $uri->dn;
+    ($ldapopts->{attr}) = $uri->attributes;
+    my %extn = $uri->extensions;
+    $ldapopts->{binddn} = $extn{binddn};
+    $ldapopts->{bindpw} = $extn{bindpw};
+
+    return 0;
+}
+
+sub _searchLdap
+{
+    my $self = shift;
+    my ($ldapopts, $filter) = @_;
+    my $base = $ldapopts->{base};
+    my $scope = 'sub';
+    my $msg;
+    my $rc;
+    my @values = ('');
+
+    if (defined($ldapopts->{ldap})) {
+        my $cmsg = $ldapopts->{ldap}->bind($ldapopts->{binddn}, password => $ldapopts->{bindpw});
+        if ($cmsg->code) {
+            $self->log(level => 'error', message => "LDAP connection check failed in handler");
+            $ldapopts->{ldap}->unbind();
+            undef($ldapopts->{ldap});
+        }
+    }
+
+    if (!$ldapopts->{ldap}) {
+        $ldapopts->{ldap} = Net::LDAP->new($ldapopts->{uri});
+        if (!defined($ldapopts->{ldap})) {
+            $self->log(level => 'error', message => "Can't connect $ldapopts->{uri}");
+            return undef;
+        }
+
+        $msg = $ldapopts->{ldap}->bind($ldapopts->{binddn}, password => $ldapopts->{bindpw});
+        $rc = $msg->code;
+        if ($rc) {
+            $self->log(level => 'error', message => "bind by $ldapopts->{binddn}failed($rc)");
+            return undef;
+        }
+    }
+
+    if ($filter =~ /^\(?dn=/) {
+        ($base) = ($filter =~ /^\(?dn=(.*)\)?$/);
+        $scope = 'base';
+        $filter = 'objectClass=*';
+    }
+
+    $msg = $ldapopts->{ldap}->search(base => $base, scope => $scope, filter => $filter, attrs => [$ldapopts->{attr}]);
+
+    $rc = $msg->code;
+    if ($rc) {
+        $self->log(level => 'error', message => "search by $filter failed($rc)");
+        if ($rc == LDAP_SERVER_DOWN) {
+            $ldapopts->{ldap}->unbind();
+            undef($ldapopts->{ldap});
+        }
+
+        return undef;
+    }
+
+    if ($msg->count) {
+        my $entry = $msg->entry(0);
+        if ($ldapopts->{attr} eq 'dn') {
+            $values[0] = $entry->dn;
+        } else {
+            @values = $entry->get_value($ldapopts->{attr});
+        }
+
+        if (!defined($values[0])) {
+            @values = ('');
+        }
+    }
+
+    return @values;
+}
+
 sub _unique
 {
     my $self = shift;
