@@ -162,8 +162,6 @@ sub _objSearch
             chop;
             my @data = split(/$dlm/);
             my $entry;
-            my $rdn_val = $data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]];
-            $rdn_val =~ s/ *$//;
 
             # check the number of returned entries
             if ($sizeLim >= 0 && @match_entries == $sizeLim) {
@@ -181,7 +179,6 @@ sub _objSearch
             }
 
             # get all values of the entry
-            my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             foreach my $oc (@{$oconf->{oc}}) {
                 $entry = $entry."objectclass: $oc\n";
             }
@@ -204,9 +201,10 @@ sub _objSearch
                 }
             }
 
-            $rdn = decode($conf->{mbcode}[0], $rdn);
             $entry = decode($conf->{mbcode}[0], $entry);
 
+            my ($rdn_val) = ($entry =~ /^$oconf->{rdn}[0]: (.*)$/mi);
+            my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             $entry = "dn: $rdn,$suffix\n$entry";
 
             # parse filter
@@ -242,9 +240,10 @@ sub _objModify
     my $rc = LDAP_SUCCESS;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
+    $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
     # multibyte string
-    $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
+    $rdn_val = encode($conf->{mbcode}[0], $rdn_val);
 
     my $lock;
     my $file;
@@ -323,10 +322,16 @@ sub _objModify
                     }
                 }
 
+                if (grep(/\r/, @values)) {
+                    $self->log(level => 'error', message => "CSV field can't contain carriage return");
+                    $rc = LDAP_UNWILLING_TO_PERFORM;
+                    last;
+                }
+
                 # multibyte value
                 for (my $i = 0; $i < @values; $i++) {
                     if ($values[$i]) {
-                        $values[$i] = encode($conf->{mbcode}[0], decode('utf8', $values[$i]));
+                        $values[$i] = encode($conf->{mbcode}[0], $values[$i]);
                     }
                 }
 
@@ -348,8 +353,10 @@ sub _objModify
 
                 if($action eq "ADD") {
                     # check whether the value already exists
-                    foreach my $val (@values) {
-                        if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$val *$valdlm/i) {
+                    for (my $i = 0; $i < @values; $i++) {
+                        my $value = $values[0];
+                        $value =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
+                        if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$value *$valdlm/i) {
                             $rc = LDAP_TYPE_OR_VALUE_EXISTS;
                             last DO;
                         }
@@ -362,10 +369,12 @@ sub _objModify
                 } elsif($action eq "DELETE") {
                     if (@values && $values[0]) {
                         # check whether the value exists
-                        foreach my $val (@values) {
-                            if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$val *$valdlm/i) {
+                        for (my $i = 0;  $i < @values; $i++) {
+                            my $value = $values[0];
+                            $value =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
+                            if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$value *$valdlm/i) {
                                 my $str = "$valdlm$data[$coln]$valdlm";
-                                $str =~ s/$valdlm$val$valdlm/$valdlm/i;
+                                $str =~ s/$valdlm$value$valdlm/$valdlm/i;
                                 ($data[$coln]) = ($str =~ /^$valdlm(.*)$valdlm$/);
                             } else {
                                 $rc = LDAP_NO_SUCH_ATTRIBUTE;
@@ -417,10 +426,11 @@ sub _objAdd
     my $rc = LDAP_SUCCESS;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
+    $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
     # multibyte string
-    $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
-    $entryStr = encode($conf->{mbcode}[0], decode('utf8', $entryStr));
+    $rdn_val = encode($conf->{mbcode}[0], $rdn_val);
+    $entryStr = encode($conf->{mbcode}[0], $entryStr);
 
     my $lock;
     my $file;
@@ -487,6 +497,11 @@ sub _objAdd
             foreach my $attr (keys %{$oconf->{attr}}) {
                 my $coln;
                 my @values = ($entryStr =~ /^$attr:\s(.*)$/gmi);
+                if (grep(/\r/, @values)) {
+                    $self->log(level => 'error', message => "CSV field can't contain carriage return");
+                    $rc = LDAP_UNWILLING_TO_PERFORM;
+                    last DO;
+                }
 
                 if (defined($oconf->{attr}{$attr}->{column})) {
                     $coln = $oconf->{attr}{$attr}->{column}[0];
@@ -555,9 +570,10 @@ sub _objDelete
     my $rc = LDAP_NO_SUCH_OBJECT;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
+    $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
     # multibyte string
-    $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
+    $rdn_val = encode($conf->{mbcode}[0], $rdn_val);
 
     my $lock;
     my $file;
