@@ -807,13 +807,15 @@ sub _objDelete
         my $sql;
         my $sth;
 
-        # delete the values from the attribute's table
-        foreach my $attr (keys %{$oconf->{attr}}) {
-            if (defined($oconf->{attr}{$attr}) && defined($oconf->{attr}{$attr}->{delproc})) {
-                if ($rc = $self->_delAttrValues($oconf, $key, $pkey, $attr, '')) {
-                    $rc = LDAP_OPERATIONS_ERROR;
-                    last DO;
-                }
+        if (!defined($conf->{nodelete}) || $conf->{nodelete} ne 'on') {
+            # delete the values from the attribute's table
+            foreach my $attr (keys %{$oconf->{attr}}) {
+                if (defined($oconf->{attr}{$attr}) && defined($oconf->{attr}{$attr}->{delproc})) {
+                    if ($rc = $self->_delAttrValues($oconf, $key, $pkey, $attr, '')) {
+                        $rc = LDAP_OPERATIONS_ERROR;
+                        last DO;
+                    }
+                 }
             }
         }
 
@@ -842,6 +844,10 @@ sub _objDelete
             }
         }
 
+        if (defined($conf->{nodelete}) && $conf->{nodelete} eq 'on') {
+            last DO;
+        }
+
         # delete the link with container
         if (defined($oconf->{container}) && defined($oconf->{container}[0]->{delproc})) {
             $sql = $oconf->{container}[0]->{delproc}[0];
